"""Error functions for ccautils module."""
import sys


def formatErrorMsg(funcname, exc):
    """Formatter for error messages.

    Args:
        funcname: str the function name where the error occurred
        exc: Exception

    Returns:
        str:
    """
    ename = type(exc).__name__
    return f"Error in {funcname}: {ename}: {exc}\n"


def errorExit(funcname, exc, errorvalue=1):
    """Exit the program displaying the error text.

    Args:
        funcname: str function name where the error occurred
        exc: Exception
        errorvalue: int exit value
    """
    sys.stderr.write(formatErrorMsg(funcname, exc))
    sys.exit(errorvalue)


def errorRaise(funcname, exc):
    """Reraises the exeption.

    Args:
        funcname: str function name where the error occurred
        exc: Exception

    Raises:
        exc: Exception
    """
    sys.stderr.write(formatErrorMsg(funcname, exc))
    raise (exc)


def errorNotify(funcname, exc):
    """Displays the exception text.

    Args:
        funcname: str function name where the error occurred
        exc: Exception
    """
    sys.stderr.write(formatErrorMsg(funcname, exc))



# Copyright (c) 2018, Christopher Allison
#
#     This file is part of ccautils.
#
#     ccautils is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     ccautils is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with ccautils.  If not, see <http://www.gnu.org/licenses/>.
"""Miscellaneous file utility functions.

a set of file based utilities for python programmes and scripts
"""
import hashlib
import os
from pathlib import Path
import sys

from ccautils.errors import errorRaise
import ccautils.utils as UT


def fileExists(fqfn):
    """Tests for the existance of a file.

    Args:
        fqfn: str fully-qualified file name

    Returns:
        Bool: True or False
    """
    try:
        return Path(fqfn).is_file()
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def dirExists(fqdn):
    """Tests for the existance of a directory.

    Args:
        fqdn: fully-qualified directory name

    Returns:
        Bool: True or False
    """
    try:
        return Path(fqdn).is_dir()
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def dfExists(fqdfn):
    """Tests for the existance of a directory or file.

    Args:
        fqdfn: fully-qualified directory or file name

    Returns:
        Bool: True or False
    """
    try:
        ret = fileExists(fqdfn)
        if not ret:
            ret = dirExists(fqdfn)
        return ret
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def makePath(pn):
    """Makes the path.

    Args:
        pn: the fully-qualified path to make
    """
    try:
        if not dfExists(pn):
            p = Path(pn)
            p.mkdir(mode=0o755, parents=True, exist_ok=True)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def makeFilePath(fqfn):
    """Makes the path for the file.

    Args:
        fqfn: fully-qualified file name
    """
    try:
        pfn = os.path.basename(fqfn)
        makePath(pfn)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def absPath(fn):
    """Transforms the filename into a fully-qualified file name.

    Args:
        fn: file name containing possible unix filesystem 'markers'

    Returns:
        str:
    """
    try:
        return os.path.abspath(os.path.expanduser(fn))
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def rename(src, dest):
    """Renames src to dest.

    Args:
        src: source file name
        dest: destination file name

    Raises:
        Exception: if source filename does not exist
    """
    try:
        if dfExists(src):
            p = Path(src)
            p.rename(dest)
        else:
            raise Exception(f"src file does not exist: {src}")
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def fileDelete(fqfn):
    """Deletes the named file.

    Args:
        fqfn: - fully-qualified filename to delete
    """
    try:
        if fileExists(fqfn):
            os.unlink(fqfn)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def fileSize(fqfn):
    """Retrieves the size of the file in bytes.

    Args:
        fqfn: str fully-qualified filename

    Returns:
        int: the size of the file
    """
    try:
        if fileExists(fqfn):
            return os.stat(fqfn).st_size
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def sizeof_fmt(num, suffix="B"):
    """Displays the size of num in human readable units.

    from article by Fred Cirera:
    https://web.archive.org/web/20111010015624/http://blogmag.net/ \
            blog/read/38/Print_human_readable_file_size
    and stackoverflow:
    https://stackoverflow.com/questions/1094841/ \
            reusable-library-to-get-human-readable-version-of-file-size

    Args:
        num: int a number (e.g. the size of a file)
        suffix: str a suffix to append to the output string

    Returns:
        str: num expressed in human readable units
    """
    try:
        for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
            if abs(num) < 1024.0:
                return f"{num:3.1f}{unit}{suffix}"
            num /= 1024.0
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def getFileHash(fqfn, blocksize=65536):
    """Compute the file sha256 hash and size.

    Args:
        fqfn: str fully-qualified file name
        blocksize: int the block size to pass to the hashing function

    Returns:
        tuple: (hash: str, filesize: int)
    """
    try:
        fnsize = fileSize(fqfn)
        sha = hashlib.sha256()
        with open(fqfn, "rb") as ifn:
            fbuf = ifn.read(blocksize)
            while len(fbuf) > 0:
                sha.update(fbuf)
                fbuf = ifn.read(blocksize)
        return (sha.hexdigest(), fnsize)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def fileTouch(fqfn, truncate=True):
    """Creates an empty file, or truncates an existing one as required.

    it is truncated if `truncate` is True and
    the file exists already, otherwise the
    access timestamp is just updated

    Args:
        fqfn: str fully-qualified filename
        truncate: Bool
    """
    try:
        if fileExists(fqfn) and truncate:
            open(fqfn, "w").close()
        elif fileExists(fqfn):
            open(fqfn, "r").close()
        else:
            open(fqfn, "w").close()
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def readFile(fqfn):
    """Reads the file into the output string.

    Args:
        fqfn: str fully-qualified filename

    Returns:
        str: or None if file does not exist
    """
    try:
        if os.path.exists(fqfn):
            with open(fqfn, "r") as ifn:
                lines = ifn.readlines()
            return UT.addToString("", lines)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)



# Copyright (c) 2018, Christopher Allison
#
#     This file is part of ccautils.
#
#     ccautils is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     ccautils is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with ccautils.  If not, see <http://www.gnu.org/licenses/>.
"""Miscellaneous utility functions.

a set of utilities for python programmes and scripts
"""
import datetime
from dateutil.relativedelta import relativedelta
import sys
import time

from ccautils.errors import errorRaise


def addToString(xstr, xadd):
    """Appends the string `xadd` to the string `xstr`.

    if xadd is a list then each list member that is a string
    is appended to xstr

    Args:
        xstr: str input string
        xadd: list or str

    Raises:
        TypeError: Exception

    Returns:
        str:
    """
    try:
        if type(xstr) is str:
            op = xstr
        else:
            op = ""
        if type(xadd) is list:
            for xi in xadd:
                if type(xi) is str:
                    op += xi
        elif type(xadd) is str:
            op += xadd
        else:
            raise TypeError("Input format error. xadd is neither list nor string")
        return op
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def delimitString(xstr, delimeter=" - "):
    """Delimits the string with the delimiter.

    Args:
        xstr: str or list
        delimeter: str

    Raises:
        ValueError: Exception

    Returns:
        str:
    """
    try:
        op = ""
        xlist = None
        if type(xstr) is str:
            xlist = xstr.split(" ")
        elif type(xstr) is list:
            xlist = xstr
        if xlist is None:
            raise ValueError("delimitString: parameter must be string or list")
        for xl in xlist:
            if len(op) > 0:
                op += delimeter + xl
            else:
                op = xl
        return op
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def makeDictFromString(istr):
    """Makes a dictionary from a string of parameters.

    leading and trailing white space is stripped

    Args:
        istr: str
            'someparam= somevalue,someotherparam =someothervalue  '

    Returns:
        dict: {"someparam": "somevalue", "someotherparam": "someothervalue"}
    """
    try:
        pd = {}
        if "=" in istr:
            ea = istr.split(",")
            for p in ea:
                tmp = p.split("=")
                pd[tmp[0].strip()] = tmp[1].strip()
        return pd
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def askMe(q, default):
    """Input routine for the console.

    Args:
        q: str input question
        default: str default answer

    Raises:
        TypeError: if input `q` is not a string

    Returns:
        str: user input or default
    """
    try:
        if type(q) is not str:
            raise TypeError("Input error, question is not a string.")
        ret = default
        val = input(f"{q} ({default}) > ")
        if len(val) > 0:
            ret = val
        return ret
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def padStr(xstr, xlen=2, pad=" ", padleft=True):
    """Pads the input string to be the required length.

    Args:
        xstr: str the input string
        xlen: int the required length
        pad: str the character or characters to pad with
        padleft: Bool

    Returns:
        str: the input string padded to the required length with pad
    """
    try:
        zstr = xstr
        while len(zstr) < xlen:
            if padleft:
                zstr = pad + zstr
            else:
                zstr += pad
        return zstr
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def reduceTime(unit, secs):
    """Performs the modulo function on secs.

    Args:
        unit: int the divisor
        secs: int a number

    Raises:
        ValueError: Exception

    Returns:
        tuple: (units: int, remainder: int)
    """
    try:
        rem = units = 0
        if unit > 0:
            units = int(secs / unit)
            rem = int(secs % unit)
        else:
            raise ValueError(
                f"divide by zero requested in reduceTime: unit: {unit}, secs: {secs}"
            )
        return (units, rem)
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def displayValue(val, label, zero=True):
    """Pluralises the label.

    if zero is True val == 0 then the empty string is returned.

    Args:
        val: number
        label: str
        zero: Bool

    Raises:
        TypeError: Exception if val is not numeric

    Returns:
        str:
    """
    try:
        if zero and val == 0:
            return ""
        dlabel = label if val == 1 else label + "s"
        sval = str(val)
        if not sval.isnumeric():
            raise TypeError("input is not numeric")
        return addToString(sval, [" ", dlabel])
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def secondsFromHMS(shms):
    """Convert "01:02:32.47" to seconds.

    Args:
        shms: str input string seperated by colons

    Returns:
        int: number of seconds represented by input string
    """
    try:
        hrs = mins = secs = extra = 0
        xtmp = shms.split(".")
        if int(xtmp[1]) > 50:
            extra = 1
        tmp = xtmp[0].split(":")
        cn = len(tmp)
        if cn == 3:
            hrs = int(tmp[0])
            mins = int(tmp[1])
            secs = int(tmp[2])
        elif cn == 2:
            mins = int(tmp[0])
            secs = int(tmp[1])
        else:
            secs = int(tmp[0])
        return (hrs * 3600) + (mins * 60) + secs + extra
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def decomplexifyhms(tim, index, labels, labindex, oplen, colons=False):
    """Secondary function to remove some of the complexity of the hms function.

    Do not call this function directly

    Args:
        tim: list
        index: int
        labels: list
        labindex: int
        oplen: int
        colons: Bool

    Returns:
        list:
    """
    try:
        op = []
        if colons:
            delim = ":"
        else:
            delim = " " if labindex == 2 else ", "
            if index == 3:
                delim = " " if labindex == 2 else " and "
        if oplen > 0:
            op.append(delim)
        if colons:
            sval = padStr(str(tim[index]), pad="0")
        else:
            if labindex == 2:
                sval = str(tim[index]) + labels[labindex][index]
            else:
                sval = displayValue(tim[index], labels[labindex][index], zero=False)
        op.append(sval)
        return op
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def hms(secs, small=True, short=True, single=False, colons=False):
    """Convert `secs` to days, hours, minutes and seconds.

    if `small` is True then only return the higher values
    if they are > zero

    if `short` is True then the labels are their short form

    if `single` is True then the labels are single letters

    if `colons` is True then the output is of the form:
         01:03:23

    Args:
        secs: int the number of seconds
        small: Bool do not return day, hours or mins if they are zero
        short: Bool use short labels
        single: Bool use single letter labels
        colons: Bool return a string of the form 01:32:24

    Returns:
        str:
    """
    try:
        labs = [
            ["day", "hour", "minute", "second"],
            ["day", "hour", "min", "sec"],
            ["d", "h", "m", "s"],
        ]

        tim = [0, 0, 0, 0]
        units = [60 * 60 * 24, 60 * 60, 60]
        rem = secs
        for index in range(3):
            tim[index], rem = reduceTime(units[index], rem)
        tim[3] = rem
        op = []
        started = not small
        if single:
            cnlabs = 2
        else:
            cnlabs = 1 if short else 0
        for cn in range(4):
            if not started and tim[cn] > 0:
                started = True
            if started:
                op += decomplexifyhms(tim, cn, labs, cnlabs, len(op), colons)
        msg = addToString("", op)
        return msg
    except Exception as e:
        fname = sys._getframe().f_code.co_name
        errorRaise(fname, e)


def timestampFromDatetime(dt):
    ts = datetime.datetime.timestamp(dt)
    return ts


def tsFromDt(dt):
    return timestampFromDatetime(dt)


def fuzzyExpires(dt):
    ts = timestampFromDatetime(dt)
    now = int(time.time())
    op = ""
    then = datetime.datetime.fromtimestamp(ts)
    now = datetime.datetime.fromtimestamp(now)
    diff = relativedelta(then, now)
    if diff.years > 0:
        tstr = displayValue(diff.years, "year")
        op = addToString(op, tstr)
        tstr = displayValue(diff.months, "month")
        op = addToString(op, [" ", tstr])
    elif diff.months > 0:
        tstr = displayValue(diff.months, "month")
        op = addToString(op, tstr)
        tstr = displayValue(diff.days, "day")
        op = addToString(op, [" ", tstr])
    elif diff.days > 0:
        tstr = displayValue(diff.days, "day")
        op = addToString(op, tstr)
        tstr = displayValue(diff.hours, "hour")
        op = addToString(op, [" ", tstr])
    elif diff.hours > 0:
        tstr = displayValue(diff.hours, "hour")
        op = addToString(op, tstr)
        tstr = displayValue(diff.minutes, "minute")
        op = addToString(op, [" ", tstr])
        tstr = displayValue(diff.seconds, "second")
        op = addToString(op, [" ", tstr])
    else:
        op = "EXPIRED"
    return (ts, op)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ccautils']

package_data = \
{'': ['*']}

install_requires = \
['python-dateutil>=2.8.1,<3.0.0', 'toml>=0.10.0,<0.11.0']

setup_kwargs = {
    'name': 'ccautils',
    'version': '0.4.0',
    'description': 'A set of useful utilities for python programs',
    'long_description': '[![Tests](https://github.com/ccdale/ccautils/workflows/Tests/badge.svg)](https://github.com/ccdale/ccautils/actions?workflow=Tests)\n[![Codecov](https://codecov.io/gh/ccdale/ccautils/branch/master/graph/badge.svg)](https://codecov.io/gh/ccdale/ccautils)\n[![Python](https://img.shields.io/pypi/pyversions/ccautils)](https://pypi.org/project/ccautils/)\n[![PyPI](https://img.shields.io/pypi/v/ccautils)](https://pypi.org/project/ccautils/)\n# ccautils\n\na set of utilities for python3.6+ programs and scripts.\n\n<a name=headdd></a>\n* [Install](#install)\n* [Development](#devel)\n* [Testing](#testing)\n* [Error Utilities](#errors)\n* [Miscellaneous Utilities](#utils)\n    * [Usage](#uusage)\n* [File Utilities](#futils)\n    * [Usage](#fusage)\n\n\n<a name=install></a>\n## [Install](#headdd)\n\nInstall for the user:\n```\npip3 install ccautils --user\n```\n\nInstall for a virtual environment:\n```\npip install ccautils\n```\n\n<a name=devel></a>\n## [Development](#headdd)\n\nI use [poetry](https://python-poetry.org/) to manage these utilities.\nClone this repository and install `poetry`, then install the dependancies.\n\n```\ngit clone https://github.com/ccdale/ccautils.git\ncd ccautils\npoetry install\n```\n\n<a name=testing></a>\n## [Testing](#headdd)\nTo run the tests you must have `pytest`, `nox` and `poetry` installed.\n\ninstall nox into your python user environment.\n```\npip install nox --user\n```\n\nRun the tests with\n\n```\nnox -rs tests\n```\n\nRun the linter with\n\n```\nnox -rs lint\n```\n\nRun the console ask tests with\n\n```\nnox -rs tests -- -sm ask\n```\n\n<a name=errors></a>\n## Error Utilities(#headdd)\n\nSee the [code](https://github.com/ccdale/ccautils) for how to use these\nException helpers.\n\n\n<a name=utils></a>\n## [Miscellaneous Utilities](#headdd)\n\n<a name=uusage></a>\n### [Usage](#headdd)\n\n```\nimport ccautils.utils as UT\n```\n\n<a name=menu></a>\n* [addToString](#addtostring)\n* [delimitString](#delimitstring)\n* [makeDictFromString](#makedictfromstring)\n* [askMe](#askme)\n* [padStr](#padstr)\n* [reduceTime](#reducetime)\n* [displayValue](#displayvalue)\n* [secondsFromHMS](#secondsfromhms)\n* [hms](#hms)\n\n<a name=addtostring></a>\n### [addToString(xstr, xadd)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L26)\n\nReturns a string with `xadd` appended to `xstr`.  If `xadd` is a list, all\n`str` members of the list will be appended in order.\n\n```\nUT.addToString("hello", [" ", "world"])\n\n> "hello world"\n```\n\n<a name=delimitstring></a>\n### [delimitString(xstr, delimeter=" - ")](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L49)\n\n`xstr` can be a list or a string.  If it is a string, it is spit apart at\nspaces and delimeted with `delimeter`.  If it is a list, each member is\ndelimeted with `delimeter`.\n\n```\nUT.delimitString(["bright", "world"], " ")\n\n> "bright world"\n\nUT.delimitString("I wandered lonely as an artichoke", ".")\n\n> "I.wandered.lonely.as.an.artichoke"\n```\n\n<a name=makedictfromstring></a>\n### [makeDictFromString(istr)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L64)\n\nConstructs a dictionary from a string of parameters. Leading and trailing\nwhitespace is stripped.\n\n`istr` should be in the form `someparam=somevalue,someotherparam=otherval`\n\n```\nUT.makeDictFromString("sparam=sval, soparam = soval")\n\n> {"sparam": "sval", "soparam": "soval"}\n```\n\n<a name=askme></a>\n### [askMe(q, default)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L89)\n\nRequests input from the user.  Poses the question `q`. Returns the users\ninput or `default` if no input given.\n\n```\nUT.askMe("press 5, please", "8")\n\n> press 5, please: 5\n> 5\n```\n\n<a name=padstr></a>\n### [padStr(xstr, xlen=2, pad=" ", padleft=True)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L101)\n\nReturns `xstr` `pad`ded to the required length, either on the\nleft (`padleft` is True) or the right (`padleft` is False)\n\n```\nUT.padStr("23", 5, "0")\n\n> "00023"\n```\n\n<a name=reducetime></a>\n### [reduceTime(unit, secs)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L115)\n\nDivides `secs` by `unit` returning a tuple of (`units`, `remainder`)\n\nRaises a `ValueError` if `unit` is zero.\n\n```\nUT.reduceTime(3600, 3700)\n\n> (1, 100)\n```\n\n<a name=displayvalue></a>\n### [displayValue(val, label, zero=True)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L131)\n\nPluralises `label` if `val` > 1 or `val` is 0.\n\nWill return an empty string if `val` == 0 and `zero` == True\n\n```\nUT.displayValue(12, "table")\n\n> "12 tables"\n```\n\n<a name=secondsfromhms></a>\n### [secondsFromHMS(shms)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L142)\n\nconverts HMS strings into integer seconds\n\n```\nUT.secondsFromHMS("01:01:23.43")\n# 1 hour, 1 minute, 23 seconds + 0.43 second\n\n> 3683\n```\n\n<a name=hms></a>\n### [hms(secs, small=True, short=True, single=False, colons=False)](#menu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/utils.py#L214)\n\nConvert `secs` to days, hours, minutes and seconds\n\nif `small` is True then only return the higher values if they are > zero\n\nif `short` is True then the labels are their short form\n\nif `single` is True then the labels are single letters\n\nif `colons` is True then the output is of the form `01:03:23`\n\n```\nUT.hms(67)\n\n> "1 min and 7 secs"\n\nUT.hms(67, short=False)\n\n> "1 minute and 7 seconds"\n\nUT.hms(67, small=False, short=False)\n\n> "0 days, 0 hours, 1 minute and 7 seconds"\n\nsecs = 86400 + 7200 + 300 + 34\nUT.hms(secs, single=True)\n\n> "1d 2h 5m 34s"\n\nsecs = 345\nUT.hms(secs, colons=True)\n\n> "05:45"\n\nsecs = 86400 + 7200 + 300 + 34\nUT.hms(secs, colons=True)\n\n> "01:02:05:34"\n```\n\n<a name=futils></a>\n## [File Utilities](#headdd)\n\n<a name=fusage></a>\n### [Usage](#headdd)\n\n```\nimport ccautils.fileutils as FT\n```\n\n<a name=fmenu></a>\n* [fileExists](#fileexists)\n* [dirExists](#direxists)\n* [dfExists](#dfexists)\n* [makePath](#makepath)\n\n<a name=fileexists></a>\n### [fileExists(fqfn)](#fmenu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/fileutils.py#L30)\n\nTests for the existence of the fully-qualified (absolute) file name `fqfn`\n\nReturns: `True` if `fqfn` exists, else `False`\n\n```\nfn = "/home/chris/output.csv"\nif FT.fileExists(fn):\n    # do something\nelse:\n    raise(f"File {fn} does not exist")\n```\n\n<a name=direxists></a>\n### [dirExists(fqdn)](#fmenu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/fileutils.py#L38)\n\nTests for the existence of the fully-qualified (absolute) directory name `fqdn`\n\nReturns: `True` if `fqdn` exists, else `False`\n\n```\ndn = "/home/chris"\nif FT.dirExists(dn):\n    # do something\nelse:\n    raise(f"Directory {dn} does not exist")\n```\n\n<a name=dfexists></a>\n### [dfExists(fqdfn)](#fmenu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/fileutils.py#L46)\n\nTests to see if the file `fqdfn` exists, if not checks if `fqdfn` is\na directory that exists.\n\nReturns: `True` if `fqdfn` exists, else `False`\n\n```\ndn = "/home/chris"\nif FT.dfExists(dn):\n    # do something\nelse:\n    raise(f"File / Directory {dn} does not exist")\n```\n\n<a name=makepath></a>\n### [makePath(pn)](#fmenu)\n\n[Code](https://github.com/ccdale/ccautils/blob/master/ccautils/fileutils.py#L57)\n\nMakes the path `pn` including any missing parent directories.  Does\nnothing if path `pn` already exists.\n\nReturns: None\n```\ndn = "/home/chris/appdir/subdir"\nFT.makePath(dn)\n```\n\n\n[modeline]: # ( vim: set ft=markdown tw=74 fenc=utf-8 spell spl=en_gb mousemodel=popup: )\n',
    'author': 'ccdale',
    'author_email': 'chris.charles.allison+ccautils@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
